//go:build !remote

package compat

import (
	"fmt"
	"net/http"

	"github.com/containers/image/v5/docker"
	"github.com/containers/image/v5/docker/reference"
	"github.com/containers/image/v5/manifest"
	"github.com/containers/podman/v5/pkg/api/handlers/utils"
	registrytypes "github.com/docker/docker/api/types/registry"
	ocispec "github.com/opencontainers/image-spec/specs-go/v1"
)

func InspectDistribution(w http.ResponseWriter, r *http.Request) {
	w.Header().Set("Content-Type", "application/json")

	name := utils.GetName(r)

	ref, err := reference.ParseAnyReference(name)
	if err != nil {
		utils.Error(w, http.StatusBadRequest, fmt.Errorf("error parsing image reference %q: %w", name, err))
		return
	}

	namedRef, ok := ref.(reference.Named)
	if !ok {
		if _, ok := ref.(reference.Digested); ok {
			// full image ID
			utils.Error(w, http.StatusBadRequest, fmt.Errorf("no manifest found for full image ID"))
			return
		}
		utils.Error(w, http.StatusBadRequest, fmt.Errorf("unknown image reference format: %s", name))
		return
	}

	namedRef = reference.TagNameOnly(namedRef)

	imgRef, err := docker.NewReference(namedRef)
	if err != nil {
		utils.Error(w, http.StatusInternalServerError, fmt.Errorf("error creating image reference: %w", err))
		return
	}

	imgSrc, err := imgRef.NewImageSource(r.Context(), nil)
	if err != nil {
		utils.Error(w, http.StatusInternalServerError, fmt.Errorf("error getting image source: %w", err))
		return
	}
	defer imgSrc.Close()

	manBlob, manType, err := imgSrc.GetManifest(r.Context(), nil)
	if err != nil {
		utils.Error(w, http.StatusInternalServerError, fmt.Errorf("error getting manifest: %w", err))
		return
	}

	digest, err := manifest.Digest(manBlob)
	if err != nil {
		utils.Error(w, http.StatusInternalServerError, fmt.Errorf("error getting manifest digest: %w", err))
		return
	}

	// todo: handle other manifest types
	ociIndex, err := manifest.OCI1IndexFromManifest(manBlob)
	if err != nil {
		utils.Error(w, http.StatusInternalServerError, fmt.Errorf("error getting OCI index from manifest: %w", err))
		return
	}

	platforms := make([]ocispec.Platform, len(ociIndex.Manifests))
	for i, m := range ociIndex.Manifests {
		platforms[i] = *m.Platform
	}

	distributionInspect := registrytypes.DistributionInspect{
		Descriptor: ocispec.Descriptor{
			Digest:    digest,
			Size:      int64(len(manBlob)),
			MediaType: manType,
		},
		Platforms: platforms,
	}

	utils.WriteResponse(w, http.StatusOK, distributionInspect)
}
